#include "plot.h"

#include <cairo.h>
#include <math.h>

#include "geom.h"

#define nlabel_colors (4)
static const double label_colors[3 * nlabel_colors] = {
  1, 0, 0,
  0, 0, 1,
  1, 0, 1,
  0.5, 0.5, 0
};

static double plot_setup(
  cairo_surface_t *surf, cairo_t *cr, double width, double height,
  size_t ndims, size_t n, const double *p, size_t k, const double *C)
{
  double min[ndims], max[ndims], size[ndims], inner_width, inner_height, start_ratio, end_ratio, scale;
  size_t i, j;

  // Find bounding box
  geom_bounding_box(ndims, n, p, min, max);
  if(C != NULL) {
    for(i = 0; i < k; i++) {  // TODO: ignore empty centers
      for(j = 0; j < ndims; j++) {
        if(C[i*ndims + j] < min[j]) {
          min[j] = C[i*ndims + j];
        }
        if(C[i*ndims + j] > max[j]) {
          max[j] = C[i*ndims + j];
        }
      }
    }
  }

  for(i = 0; i < ndims; i++) {
    min[i] -= (max[i] - min[i]) / 10;
    max[i] += (max[i] - min[i]) / 10;
  }

  // Rescale
  for(i = 0; i < ndims; i++) {
    size[i] = max[i] - min[i];
  }
  start_ratio = (double)width/height;
  end_ratio = size[0]/size[1];
  
  if(end_ratio < start_ratio) {
    // height is limiting dimension
    scale = height / size[1];

    inner_width = end_ratio * height;
    cairo_translate(cr, (width - inner_width) / 2, 0);
  }
  else {
    // width is limiting dimension
    scale = width / size[0];

    inner_height = width / end_ratio;
    cairo_translate(cr, 0, (height - inner_height) / 2);
  }

  cairo_scale(cr, scale, scale);
  cairo_set_line_width(cr, cairo_get_line_width(cr) * scale);

  cairo_translate(cr, -min[0], -min[1]);

  return scale;
}

void cluster_plot(cairo_surface_t *surf, double width, double height,
                  size_t ndims, size_t n, const double *p,
									size_t k, const int *part, const double *C)
{
  cairo_t *cr;
  size_t i, j;
  double scale;

  cr = cairo_create(surf);
  scale = plot_setup(surf, cr, width, height, ndims, n, p, k, C);

  // Draw points
  for(i = 0; i < n; i++) {
    if(part == NULL) {
      cairo_set_source_rgb(cr, 0, 0, 0);
    }
    else {
      j = ((size_t)part[i]) % nlabel_colors;
      cairo_set_source_rgb(cr, label_colors[3*j], label_colors[3*j + 1], label_colors[3*j + 2]);
    }

    cairo_arc(cr, p[ndims*i], p[ndims*i+1], 2/scale, 0, 2 * M_PI);
    cairo_fill(cr);
  }

  // Draw centers
  if(C != NULL) {
    cairo_set_source_rgb(cr, 0, 0, 0);
    cairo_set_line_width(cr, 2.5 / scale);
    for(i = 0; i < k; i++) {
      printf("%g %g\n", C[ndims*i], C[ndims*i+1]);

      cairo_move_to(cr, C[ndims*i], C[ndims*i+1] - 10 / scale);
      cairo_line_to(cr, C[ndims*i], C[ndims*i+1] + 10 / scale);
      cairo_stroke(cr);

      cairo_move_to(cr, C[ndims*i] - 10 / scale, C[ndims*i+1]);
      cairo_line_to(cr, C[ndims*i] + 10 / scale, C[ndims*i+1]);
      cairo_stroke(cr);
    }
  }

  cairo_destroy(cr);
}

static const double bg_color[3] = {1.0, 1.0, 1.0};

void cluster_plot_clear(cairo_surface_t *surf)
{
  cairo_t *cr;

  cr = cairo_create(surf);

  cairo_set_source_rgb(cr, bg_color[0], bg_color[1], bg_color[2]);
  cairo_paint(cr);

  cairo_destroy(cr);
}
