#include <time.h>
#include <stdlib.h>

#include <cairo.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

#include "geom.h"
#include "greedy_clustering.h"
#include "plot.h"

#define NPOINTS (1024)
#define NCLUSTERS (4)
#define NDIMS (2)
#define SIGMA (0.1)


static void uniform_cluster_centers(size_t ndims, size_t nclusters, double *centers);
static void cluster_sample(size_t ndims, size_t nclusters, const double *centers, size_t n, double *p, double sigma);

int main(int argc, char **argv)
{
  unsigned seed;

  const char *out_path;
  unsigned int width, height;
	cairo_surface_t *surf;
  cairo_t *cr;

  double *p, *C;
  int *part;

  seed = time(NULL);
  srand(seed);
  printf("seed = %u\n", seed);

  // Interpret arguments
  if(argc < 2) {
    out_path = "cluster.png";
  }
  else {
    out_path = argv[1];
  }

  if(argc >= 4) {
    sscanf(argv[2], "%u", &width);
    sscanf(argv[3], "%u", &height);
  }
  else if(argc >= 3) {
    sscanf(argv[2], "%u", &width);
    height = width;
  }
  else {
    width = height = 640;
  }

  // Create random problem
  C = (double *)malloc(sizeof(double) * NCLUSTERS * NDIMS);
  uniform_cluster_centers(NDIMS, NCLUSTERS, C);

  p = (double *)malloc(sizeof(double) * NPOINTS * NDIMS);
  cluster_sample(NDIMS, NCLUSTERS, C, NPOINTS, p, SIGMA);
  
  // Perform greedy clustering
  part = (int *)malloc(sizeof(int) * NPOINTS);
  greedy_cluster(NPOINTS, NDIMS, p, NCLUSTERS, -1, C, part);

  // Plot points
  surf = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
  cluster_plot_clear(surf);
  cluster_plot(surf, width,  height, NDIMS, NPOINTS, p, NCLUSTERS, part, C);

  // Save result
  cairo_surface_write_to_png(surf, out_path);

  // Cleanup
  cairo_surface_destroy(surf);
  free(part);
  free(p);
  free(C);

  return 0;
}

static inline double frand(double min, double max)
{
  return (rand() / (double)RAND_MAX) * (max-min) + min;
}

static void uniform_cluster_centers(size_t ndims, size_t nclusters, double *centers)
{
  size_t i;

  for(i = 0; i < ndims * nclusters; i++) {
    centers[i] = frand(0, 1);
  }
}

static void cluster_sample(size_t ndims, size_t nclusters, const double *centers, size_t n, double *p, double sigma)
{
  gsl_rng *rng;
  size_t i, j, k;

  // Initialize random number generator
  rng = gsl_rng_alloc(gsl_rng_default);
  gsl_rng_set(rng, rand());

  // Sample points
  for(i = 0; i < n; i++) {
    k = gsl_rng_get(rng) % nclusters;

    // Sample isotropic gaussian with mean centers[k] and std. dev. sigma
    for(j = 0; j < ndims; j++) {
      p[ndims*i + j] = gsl_ran_gaussian(rng, sigma) + centers[ndims*k + j];
    }
  }

  gsl_rng_free(rng);
}
